#include <vector>

#include "caffe/layers/deconv_hash_layer.hpp"

namespace caffe {

template <typename Dtype>
void DeconvHashLayer<Dtype>::compute_output_shape() {
	//can be got from the top hash structure
}

template <typename Dtype>
void DeconvHashLayer<Dtype>::Forward_cpu(const vector<Blob<Dtype>*>& bottom,
      const vector<Blob<Dtype>*>& top) 
{
  const Dtype* weight = this->blobs_[0]->cpu_data();
  this->backward_cpu_gemm(bottom[HASH_DATA_BLOB]->cpu_data(), weight,
	  top[HASH_DATA_BLOB]->mutable_cpu_data());

  if (this->bias_term_)
  {
	  const Dtype* bias = this->blobs_[1]->cpu_data();
	  this->forward_cpu_bias(top[HASH_DATA_BLOB]->mutable_cpu_data(), bias);
  }
}

template <typename Dtype>
void DeconvHashLayer<Dtype>::Backward_cpu(const vector<Blob<Dtype>*>& top,
      const vector<bool>& propagate_down, const vector<Blob<Dtype>*>& bottom) 
{
  const Dtype* weight = this->blobs_[0]->cpu_data();
  Dtype* weight_diff = this->blobs_[0]->mutable_cpu_diff();

  const Dtype* top_diff = top[HASH_DATA_BLOB]->cpu_diff();
  const Dtype* bottom_data = bottom[HASH_DATA_BLOB]->cpu_data();
  Dtype* bottom_diff = bottom[HASH_DATA_BLOB]->mutable_cpu_diff();
  // Bias gradient, if necessary.
  if (this->bias_term_ && this->param_propagate_down_[1])
  {
	  Dtype* bias_diff = this->blobs_[1]->mutable_cpu_diff();
	  this->backward_cpu_bias(bias_diff, top_diff);
  }
  if (this->param_propagate_down_[0] || propagate_down[HASH_DATA_BLOB])
  {
	  // gradient w.r.t. weight. Note that we will accumulate diffs.
	  if (this->param_propagate_down_[0])
	  {
		  this->weight_cpu_gemm(top_diff,
			  bottom_data, weight_diff);
	  }
	  // gradient w.r.t. bottom data, if necessary.
	  if (propagate_down[HASH_DATA_BLOB])
	  {
		  this->forward_cpu_gemm(top_diff, weight,
			  bottom_diff);
	  }
  }
}

#ifdef CPU_ONLY
STUB_GPU(DeconvHashLayer);
#endif

INSTANTIATE_CLASS(DeconvHashLayer);
REGISTER_LAYER_CLASS(DeconvHash);

}  // namespace caffe
